@@ -27,6 +27,7 @@
 #include <config.h>
 #include <string.h>
 #include <stdio.h>
+#include <locale.h>
 #include "nautilus-file-operations.h"
 
 #include "nautilus-file-operations-progress.h"
@@ -43,6 +44,7 @@
 #include <gdk/gdkdnd.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gtk/gtkwidget.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-find-directory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -57,7 +59,11 @@
 #include "nautilus-link.h"
 #include "nautilus-trash-monitor.h"
 
-typedef enum {
+typedef enum   TransferKind         TransferKind;
+typedef struct TransferInfo         TransferInfo;
+typedef struct IconPositionIterator IconPositionIterator;
+
+enum TransferKind {
 	TRANSFER_MOVE,
 	TRANSFER_COPY,
 	TRANSFER_DUPLICATE,
@@ -65,10 +71,10 @@ typedef enum {
 	TRANSFER_EMPTY_TRASH,
 	TRANSFER_DELETE,
 	TRANSFER_LINK
-} TransferKind;
+};
 
 /* Copy engine callback state */
-typedef struct {
+struct TransferInfo {
 	GnomeVFSAsyncHandle *handle;
 	NautilusFileOperationsProgress *progress_dialog;
 	const char *operation_title;	/* "Copying files" */
@@ -84,7 +90,8 @@ typedef struct {
 	gpointer done_callback_data;
 	GHashTable *debuting_uris;
 	gboolean cancelled;	
-} TransferInfo;
+	IconPositionIterator *iterator;
+};
 
 static TransferInfo *
 transfer_info_new (GtkWidget *parent_view)
@@ -119,17 +126,20 @@ transfer_info_destroy (TransferInfo *transfer_info)
  * top level items during a copy, drag, new folder creation and
  * link creation
  */
-typedef struct {
+struct IconPositionIterator {
 	GdkPoint *icon_positions;
 	int last_icon_position_index;
 	GList *uris;
 	const GList *last_uri;
 	int screen;
-} IconPositionIterator;
+	gboolean is_source_iterator;
+};
 
 static IconPositionIterator *
-icon_position_iterator_new (GArray *icon_positions, const GList *uris,
-			    int screen)
+icon_position_iterator_new (GArray *icon_positions,
+			    const GList *uris,
+			    int screen,
+			    gboolean is_source_iterator)
 {
 	IconPositionIterator *result;
 	guint index;
@@ -147,10 +157,38 @@ icon_position_iterator_new (GArray *icon_positions, const GList *uris,
 	result->uris = eel_g_str_list_copy ((GList *)uris);
 	result->last_uri = result->uris;
 	result->screen = screen;
+	result->is_source_iterator = is_source_iterator;
 
 	return result;
 }
 
+static IconPositionIterator *
+icon_position_iterator_new_single (GdkPoint *icon_position,
+				   const char *uri,
+				   int screen,
+				   gboolean is_source_iterator)
+{
+	IconPositionIterator *iterator;
+	GArray *icon_positions;
+	GList *uris;
+
+	if (icon_position == NULL || uri == NULL) {
+		return NULL;
+	}
+
+	icon_positions = g_array_sized_new (FALSE, FALSE, sizeof (GdkPoint), 1);
+	g_array_insert_val (icon_positions, 0, *icon_position);
+
+	uris = g_list_append (NULL, (char *) uri);
+
+	iterator = icon_position_iterator_new (icon_positions, uris, screen, is_source_iterator);
+
+	g_list_free (uris);
+	g_array_free (icon_positions, TRUE);
+
+	return iterator;
+}
+
 static void
 icon_position_iterator_free (IconPositionIterator *position_iterator)
 {
@@ -165,13 +203,22 @@ icon_position_iterator_free (IconPositionIterator *position_iterator)
 
 static gboolean
 icon_position_iterator_get_next (IconPositionIterator *position_iterator,
-				 const char *next_uri,
+				 const char *next_source_uri,
+				 const char *next_target_uri,
 				 GdkPoint *point)
 {
+	const char *next_uri;
+
 	if (position_iterator == NULL) {
 		return FALSE;
 	}
-		
+
+	if (position_iterator->is_source_iterator) {
+		next_uri = next_source_uri;
+	} else {
+		next_uri = next_target_uri;
+	}
+
 	for (;;) {
 		if (position_iterator->last_uri == NULL) {
 			/* we are done, no more points left */
@@ -205,6 +252,42 @@ icon_position_iterator_get_next (IconPositionIterator *position_iterator,
 	return TRUE;
 }
 
+static void
+icon_position_iterator_update_uri (IconPositionIterator *position_iterator,
+				   const char *old_uri,
+				   const char *new_uri_fragment)
+{
+	GnomeVFSURI *uri, *parent_uri;
+	GList *l;
+
+	if (position_iterator == NULL) {
+		return;
+	}
+
+	l = g_list_find_custom (position_iterator->uris,
+			        old_uri,
+				(GCompareFunc) strcmp);
+	if (l == NULL) {
+		return;
+	}
+
+	uri = gnome_vfs_uri_new (old_uri);
+	parent_uri = gnome_vfs_uri_get_parent (uri);
+	gnome_vfs_uri_unref (uri);
+
+	if (parent_uri == NULL) {
+		return;
+	}
+	
+	uri = gnome_vfs_uri_append_string (parent_uri, new_uri_fragment);
+
+	g_free (l->data);
+	l->data = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+
+	gnome_vfs_uri_unref (uri);
+	gnome_vfs_uri_unref (parent_uri);
+}
+
 static char *
 ellipsize_string_for_dialog (PangoContext *context, const char *str)
 {
@@ -831,7 +914,6 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 	char *detail;
 	char *formatted_source_name;
 	char *formatted_target_name;
-	const char *dialog_title;
 	NautilusFileOperationsErrorKind error_kind;
 	NautilusFileOperationsErrorLocation error_location;
 	
@@ -925,40 +1007,18 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 				    progress_info->vfs_status,
 				    &text, &detail);
 
-		switch (transfer_info->kind) {
-		case TRANSFER_COPY:
-		case TRANSFER_DUPLICATE:
-			dialog_title = _("Error While Copying");
-			break;
-		case TRANSFER_MOVE:
-			dialog_title = _("Error While Moving");
-			break;
-		case TRANSFER_LINK:
-			dialog_title = _("Error While Linking");
-			break;
-		case TRANSFER_DELETE:
-		case TRANSFER_EMPTY_TRASH:
-		case TRANSFER_MOVE_TO_TRASH:
-			dialog_title = _("Error While Deleting");
-			break;
-		default:
-			dialog_title = NULL;
-			break;
-		}
-
 		if (error_location == ERROR_LOCATION_TARGET ||
 		    error_kind == ERROR_SOURCE_IN_TARGET) {
 			/* We can't continue, just tell the user. */
 			eel_run_simple_dialog (parent_for_error_dialog (transfer_info),
-				TRUE, GTK_MESSAGE_ERROR, text, detail, dialog_title, GTK_STOCK_OK, NULL);
+				TRUE, GTK_MESSAGE_ERROR, text, detail, GTK_STOCK_OK, NULL);
 			error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 
 		} else if (progress_info->files_total == 1) {
 			error_dialog_button_pressed = eel_run_simple_dialog
 				(parent_for_error_dialog (transfer_info), TRUE, 
 				 GTK_MESSAGE_ERROR, text, 
-				 detail, dialog_title,
-				 GTK_STOCK_CANCEL, _("_Retry"), NULL);
+				 detail, GTK_STOCK_CANCEL, _("_Retry"), NULL);
 
 			switch (error_dialog_button_pressed) {
 			case 0:
@@ -975,8 +1035,7 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 			error_dialog_button_pressed = eel_run_simple_dialog
 				(parent_for_error_dialog (transfer_info), TRUE, 
 				 GTK_MESSAGE_ERROR, text, 
-				 detail, dialog_title,
-				 _("_Skip"), GTK_STOCK_CANCEL, _("_Retry"), NULL);
+				 detail, _("_Skip"), GTK_STOCK_CANCEL, _("_Retry"), NULL);
 
 			switch (error_dialog_button_pressed) {
 			case 0:
@@ -1144,20 +1203,22 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 			(parent_for_error_dialog (transfer_info), TRUE, GTK_MESSAGE_QUESTION, text, 
 			 secondary_text, 
 			 _("Conflict While Copying"),
-			 _("Replace _All"), _("_Skip"), _("_Replace"), NULL);
+			 _("S_kip All"), _("Replace _All"), _("_Skip"), _("_Replace"), NULL);
 		g_free (text);
 
 		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
 
 		switch (result) {
 		case 0:
-			return GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE_ALL;
+			return GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP_ALL;
 		case 1:
+			return GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE_ALL;
+		case 2:
 			return GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP;
 		default:
 			g_assert_not_reached ();
 			/* fall through */
-		case 2:
+		case 3:
 			return GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE;
 		}
 	}
@@ -1621,7 +1682,7 @@ apply_one_position (IconPositionIterator *position_iterator,
 {
 	GdkPoint point;
 
-	if (icon_position_iterator_get_next (position_iterator, source_name, &point)) {
+	if (icon_position_iterator_get_next (position_iterator, source_name, target_name, &point)) {
 		nautilus_file_changes_queue_schedule_position_set (target_name, point, position_iterator->screen);
 	} else {
 		nautilus_file_changes_queue_schedule_position_remove (target_name);
@@ -1666,10 +1727,12 @@ sync_transfer_callback (GnomeVFSXferProgressInfo *progress_info, gpointer data)
 					nautilus_file_changes_queue_schedule_metadata_copy 
 						(progress_info->source_name, progress_info->target_name);
 
-					apply_one_position (position_iterator,
-							    progress_info->source_name,
-							    progress_info->target_name);
 				}
+
+				apply_one_position (position_iterator,
+						    progress_info->source_name,
+						    progress_info->target_name);
+
 				if (debuting_uris != NULL) {
 					g_hash_table_replace (debuting_uris,
 							      g_strdup (progress_info->target_name),
@@ -1697,12 +1760,12 @@ sync_transfer_callback (GnomeVFSXferProgressInfo *progress_info, gpointer data)
 				if (really_moved) {
 					nautilus_file_changes_queue_schedule_metadata_move 
 						(progress_info->source_name, progress_info->target_name);
-					
-					apply_one_position (position_iterator,
-							    progress_info->source_name,
-							    progress_info->target_name);
 				}
-				
+
+				apply_one_position (position_iterator,
+						    progress_info->source_name,
+						    progress_info->target_name);
+
 				if (debuting_uris != NULL) {
 					g_hash_table_replace (debuting_uris,
 							      g_strdup (progress_info->target_name),
@@ -1943,7 +2006,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		 * here at all.
 		 */
 		icon_position_iterator = icon_position_iterator_new
-			(relative_item_points, item_uris, screen_num);
+			(relative_item_points, item_uris, screen_num, TRUE);
 	} else {
 		icon_position_iterator = NULL;
 	}
@@ -2106,6 +2169,8 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	sync_transfer_info->iterator = icon_position_iterator;
 	sync_transfer_info->debuting_uris = transfer_info->debuting_uris;
 
+	transfer_info->iterator = sync_transfer_info->iterator;
+
 	if (result == GNOME_VFS_OK) {
 		gnome_vfs_async_xfer (&transfer_info->handle, source_uri_list, target_uri_list,
 		      		      move_options, GNOME_VFS_XFER_ERROR_MODE_QUERY, 
@@ -2128,6 +2193,7 @@ typedef struct {
 	NautilusNewFolderCallback done_callback;
 	gpointer data;
 	GtkWidget *parent_view;
+	IconPositionIterator *iterator;
 } NewFolderTransferState;
 
 static int
@@ -2148,7 +2214,7 @@ handle_new_folder_vfs_error (const GnomeVFSXferProgressInfo *progress_info, NewF
 		error_string_to_free = (char *)error_string;
 	}
 	
-	eel_show_error_dialog (_("Error creating new folder."), error_string, _("Error Creating New Folder"),
+	eel_show_error_dialog (_("Error creating new folder."), error_string,
 				    GTK_WINDOW (gtk_widget_get_toplevel (state->parent_view)));
 	
 	g_free (error_string_to_free);
@@ -2207,6 +2273,12 @@ new_folder_transfer_callback (GnomeVFSAsyncHandle *handle,
 					 progress_info->duplicate_count);
 			}
 			g_free (temp_string);
+
+			icon_position_iterator_update_uri
+				(state->iterator,
+				 progress_info->target_name,
+				 progress_info->duplicate_name);
+
 			return GNOME_VFS_XFER_ERROR_ACTION_SKIP;
 	
 		case GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR:
@@ -2224,20 +2296,16 @@ new_folder_transfer_callback (GnomeVFSAsyncHandle *handle,
 
 void 
 nautilus_file_operations_new_folder (GtkWidget *parent_view, 
+				     GdkPoint *target_point,
 				     const char *parent_dir,
 				     NautilusNewFolderCallback done_callback,
 				     gpointer data)
 {
 	GList *target_uri_list;
 	GnomeVFSURI *uri, *parent_uri;
-	char *dirname;
+	char *text_uri, *dirname;
 	NewFolderTransferState *state;
-
-	state = g_new (NewFolderTransferState, 1);
-	state->done_callback = done_callback;
-	state->data = data;
-	state->parent_view = parent_view;
-	eel_add_weak_pointer (&state->parent_view);
+	SyncTransferInfo *sync_transfer_info;
 
 	/* pass in the target directory and the new folder name as a destination URI */
 	parent_uri = gnome_vfs_uri_new (parent_dir);
@@ -2247,14 +2315,32 @@ nautilus_file_operations_new_folder (GtkWidget *parent_view,
 	uri = gnome_vfs_uri_append_file_name (parent_uri, dirname);
 	g_free (dirname);
 	target_uri_list = g_list_prepend (NULL, uri);
-	
+
+	text_uri = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+
+	sync_transfer_info = g_new (SyncTransferInfo, 1);
+	sync_transfer_info->iterator = icon_position_iterator_new_single
+		(target_point, text_uri,
+		 gdk_screen_get_number (gtk_widget_get_screen (parent_view)),
+		 FALSE);
+	sync_transfer_info->debuting_uris = NULL;
+
+	g_free (text_uri);
+
+	state = g_new (NewFolderTransferState, 1);
+	state->done_callback = done_callback;
+	state->data = data;
+	state->parent_view = parent_view;
+	state->iterator = sync_transfer_info->iterator;
+	eel_add_weak_pointer (&state->parent_view);
+
 	gnome_vfs_async_xfer (&state->handle, NULL, target_uri_list,
 	      		      GNOME_VFS_XFER_NEW_UNIQUE_DIRECTORY,
 	      		      GNOME_VFS_XFER_ERROR_MODE_QUERY, 
 	      		      GNOME_VFS_XFER_OVERWRITE_MODE_QUERY,
 			      GNOME_VFS_PRIORITY_DEFAULT,
 	      		      new_folder_transfer_callback, state,
-	      		      sync_transfer_callback, NULL);
+	      		      sync_transfer_callback, sync_transfer_info);
 
 	gnome_vfs_uri_list_free (target_uri_list);
 	gnome_vfs_uri_unref (parent_uri);
@@ -2266,6 +2352,7 @@ typedef struct {
 	gpointer data;
 	GtkWidget *parent_view;
 	GHashTable *debuting_uris;
+	IconPositionIterator *iterator;
 } NewFileTransferState;
 
 
@@ -2287,7 +2374,7 @@ handle_new_file_vfs_error (const GnomeVFSXferProgressInfo *progress_info, NewFil
 		error_string_to_free = (char *)error_string;
 	}
 	
-	eel_show_error_dialog (_("Error creating new document."), error_string, _("Error Creating New Document"),
+	eel_show_error_dialog (_("Error creating new document."), error_string,
 			       GTK_WINDOW (gtk_widget_get_toplevel (state->parent_view)));
 	
 	g_free (error_string_to_free);
@@ -2374,6 +2461,7 @@ new_file_transfer_callback (GnomeVFSAsyncHandle *handle,
 				g_strfreev (temp_strings);
 			}
 			g_free (temp_string);
+
 			return GNOME_VFS_XFER_ERROR_ACTION_SKIP;
 	
 		case GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR:
@@ -2391,6 +2479,7 @@ new_file_transfer_callback (GnomeVFSAsyncHandle *handle,
 
 void 
 nautilus_file_operations_new_file_from_template (GtkWidget *parent_view, 
+						 GdkPoint *target_point,
 						 const char *parent_dir,
 						 const char *target_filename,
 						 const char *template_uri,
@@ -2408,18 +2497,12 @@ nautilus_file_operations_new_file_from_template (GtkWidget *parent_view,
 	g_assert (parent_dir != NULL);
 	g_assert (template_uri != NULL);
 
-	state = g_new (NewFileTransferState, 1);
-	state->done_callback = done_callback;
-	state->data = data;
-	state->parent_view = parent_view;
-
 	/* pass in the target directory and the new folder name as a destination URI */
 	parent_uri = gnome_vfs_uri_new (parent_dir);
 
 	source_uri = gnome_vfs_uri_new (template_uri);
 	if (source_uri == NULL) {
 		(*done_callback) (NULL, data);
-		g_free (state);
 		return;
 	}
 
@@ -2431,16 +2514,24 @@ nautilus_file_operations_new_file_from_template (GtkWidget *parent_view,
 		g_free (tmp);
 	}
 
-	state->debuting_uris = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	sync_transfer_info = g_new (SyncTransferInfo, 1);
+	sync_transfer_info->iterator = icon_position_iterator_new_single
+		(target_point, template_uri,
+		 gdk_screen_get_number (gtk_widget_get_screen (parent_view)),
+		 TRUE);
+	sync_transfer_info->debuting_uris = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+	state = g_new (NewFileTransferState, 1);
+	state->done_callback = done_callback;
+	state->data = data;
+	state->parent_view = parent_view;
+	state->iterator = sync_transfer_info->iterator;
+	state->debuting_uris = sync_transfer_info->debuting_uris;
 	eel_add_weak_pointer (&state->parent_view);
 
 	target_uri_list = g_list_prepend (NULL, target_uri);
 	source_uri_list = g_list_prepend (NULL, source_uri);
 
-	sync_transfer_info = g_new (SyncTransferInfo, 1);
-	sync_transfer_info->iterator = NULL;
-	sync_transfer_info->debuting_uris = state->debuting_uris;
-
 	options = GNOME_VFS_XFER_USE_UNIQUE_NAMES;
 
 	gnome_vfs_async_xfer (&state->handle, source_uri_list, target_uri_list,
@@ -2482,6 +2573,7 @@ new_file_from_temp_callback (const char *new_file_uri,
 
 void 
 nautilus_file_operations_new_file (GtkWidget *parent_view, 
+				   GdkPoint *target_point,
 				   const char *parent_dir,
 				   const char *initial_contents,
 				   NautilusNewFileCallback done_callback,
@@ -2519,6 +2611,7 @@ nautilus_file_operations_new_file (GtkWidget *parent_view,
 	new_data->callback_data = data;
 	
 	nautilus_file_operations_new_file_from_template (parent_view, 
+							 target_point,
 							 parent_dir,
 							 target_filename,
 							 source_file_uri,
