@@ -89,12 +89,17 @@ typedef char * (*NautilusTruncateCallback)    (const char    *string,
 					       int	      width,
 					       void	     *context);
 
+typedef void NautilusFileListHandle;
+
 /* GObject requirements. */
 GType                   nautilus_file_get_type                          (void);
 
 /* Getting at a single file. */
 NautilusFile *          nautilus_file_get                               (const char                     *uri);
 
+/* Get a file only if the nautilus version already exists */
+NautilusFile *          nautilus_file_get_existing                      (const char                     *uri);
+
 /* Covers for gtk_object_ref and gtk_object_unref that provide two conveniences:
  * 1) You don't have to cast to GtkObject *, so using these is type safe.
  * 2) You are allowed to call these with NULL,
@@ -167,6 +172,7 @@ NautilusRequestStatus   nautilus_file_get_deep_counts                   (Nautilu
 									 GnomeVFSFileSize               *total_size,
 									 gboolean                        force);
 gboolean                nautilus_file_should_show_directory_item_count  (NautilusFile                   *file);
+gboolean                nautilus_file_should_show_type                  (NautilusFile                   *file);
 GList *                 nautilus_file_get_keywords                      (NautilusFile                   *file);
 void                    nautilus_file_set_keywords                      (NautilusFile                   *file,
 									 GList                          *keywords);
@@ -340,8 +346,10 @@ GList *                 nautilus_file_list_copy                         (GList
 GList *			nautilus_file_list_sort_by_display_name		(GList				*file_list);
 void                    nautilus_file_list_call_when_ready              (GList                          *file_list,
 									 NautilusFileAttributes          attributes,
+									 NautilusFileListHandle        **handle,
 									 NautilusFileListCallback        callback,
 									 gpointer                        callback_data);
+void                    nautilus_file_list_cancel_call_when_ready       (NautilusFileListHandle         *handle);
 
 /* Debugging */
 void                    nautilus_file_dump                              (NautilusFile                   *file);
