@@ -387,7 +387,7 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 				       0,
 				       CopyFromParent,
 				       CopyFromParent,
-				       CopyFromParent,
+				       (Visual *)CopyFromParent,
 				       CWOverrideRedirect | CWEventMask,
 				       &attrs);
 		
@@ -628,6 +628,8 @@ void nautilus_launch_show_file (NautilusFile *file,
 			detail_message = g_strdup 
 				(_("Please check the spelling and try again."));
 			break;
+		case GNOME_VFS_ERROR_CANCELLED:
+			break;
 		case GNOME_VFS_OK:
 		default:
 #if NEW_MIME_COMPLETE
@@ -641,7 +643,7 @@ void nautilus_launch_show_file (NautilusFile *file,
 	}
 
 	if (error_message != NULL) {
-		eel_show_error_dialog (error_message, detail_message, _("Can't Display Location"), parent_window);
+		eel_show_error_dialog (error_message, detail_message, parent_window);
 		
 		g_free (error_message);
 		g_free (detail_message);
@@ -855,7 +857,9 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 	const GList *p;
 	int total, count;
 	char **envp;
+#ifdef HAVE_STARTUP_NOTIFICATION
 	Time timestamp;
+#endif
 
 	/* strip the leading command specifier */
 	if (eel_str_has_prefix (desktop_file_uri, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
@@ -873,7 +877,6 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 			(_("Sorry, but you can't execute commands from "
 			   "a remote site."), 
 			 _("This is disabled due to security considerations."),
-			 _("Can't Execute Remote Links"),
 			 parent_window);
 			 
 		return;
@@ -887,7 +890,6 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 		eel_show_error_dialog
 			(_("There was an error launching the application."),
 			 message,
-			 _("Error Launching Application"),
 			 parent_window);			
 			 
 		g_error_free (error);
@@ -918,7 +920,6 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 				(_("This drop target only supports local files."),
 				 _("To open non-local files copy them to a local folder and then"
 				   " drop them again."),
-				 _("Drop Target Only Supports Local Files"),
 				 parent_window);
 			
 			gnome_desktop_item_unref (ditem);
@@ -930,7 +931,6 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 				(_("This drop target only supports local files."),
 				 _("To open non-local files copy them to a local folder and then"
 				   " drop them again. The local files you dropped have already been opened."),
-				 _("Drop Target Only Supports Local Files"),
 				 parent_window);
 		}		
 	}
@@ -946,8 +946,10 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 
 	error = NULL;
 
+#ifdef HAVE_STARTUP_NOTIFICATION
 	timestamp = slowly_and_stupidly_obtain_timestamp (GDK_WINDOW_XDISPLAY (GTK_WIDGET (parent_window)->window));
 	gnome_desktop_item_set_launch_time (ditem, timestamp);
+#endif
 	gnome_desktop_item_launch_with_env (ditem, (GList *) parameter_uris,
 					    flags, envp,
 					    &error);
@@ -956,7 +958,6 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
 		eel_show_error_dialog
 			(_("There was an error launching the application."),
 			 message,
-			 _("Error Launching Application"),
 			 parent_window);			
 			 
 		g_error_free (error);
