@@ -60,6 +60,10 @@ struct NautilusLocationEntryDetails {
 	GList *file_info_list;
 	
 	guint idle_id;
+
+	gboolean has_special_text;
+	gboolean setting_special_text;
+	gchar *special_text;
 };
 
 static void  nautilus_location_entry_class_init       (NautilusLocationEntryClass *class);
@@ -194,7 +198,7 @@ try_to_expand_path (gpointer callback_data)
 
 	/* Trailing whitespace is OK here since the cursor is known to
 	   be at the end of the text and therefor after the whitespace. */
-	current_path = eel_make_uri_from_input_with_trailing_ws (user_location);
+	current_path = gnome_vfs_make_uri_from_input_with_trailing_ws (user_location);
 	if (!eel_istr_has_prefix (current_path, "file://")) {
 		g_free (user_location);
 		g_free (current_path);
@@ -379,11 +383,10 @@ position_and_selection_are_at_end (GtkEditable *editable)
 static void
 editable_event_after_callback (GtkEntry *entry,
 			       GdkEvent *event,
-			       gpointer user_data)
+			       NautilusLocationEntry *location_entry)
 {
 	GtkEditable *editable;
 	GdkEventKey *keyevent;
-	NautilusLocationEntry *location_entry;
 
 	if (event->type != GDK_KEY_PRESS) {
 		return;
@@ -391,7 +394,6 @@ editable_event_after_callback (GtkEntry *entry,
 
 	editable = GTK_EDITABLE (entry);
 	keyevent = (GdkEventKey *)event;
-	location_entry = NAUTILUS_LOCATION_ENTRY (user_data);
 
 	/* After typing the right arrow key we move the selection to
 	 * the end, if we have a valid selection - since this is most
@@ -433,6 +435,7 @@ finalize (GObject *object)
 
 	entry = NAUTILUS_LOCATION_ENTRY (object);
 
+	g_free (entry->details->special_text);
 	g_free (entry->details);
 
 	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
@@ -462,12 +465,43 @@ destroy (GtkObject *object)
 	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
+static void
+nautilus_location_entry_text_changed (NautilusLocationEntry *entry,
+				      GParamSpec            *pspec)
+{
+	if (entry->details->setting_special_text) {
+		return;
+	}
+
+	entry->details->has_special_text = FALSE;
+}
+
+static gboolean
+nautilus_location_entry_focus_in (GtkWidget     *widget,
+				  GdkEventFocus *event)
+{
+	NautilusLocationEntry *entry = NAUTILUS_LOCATION_ENTRY (widget);
+
+	if (entry->details->has_special_text) {
+		entry->details->setting_special_text = TRUE;
+		gtk_entry_set_text (GTK_ENTRY (entry), "");
+		entry->details->setting_special_text = FALSE;
+	}
+
+	return EEL_CALL_PARENT_WITH_RETURN_VALUE (GTK_WIDGET_CLASS, focus_in_event, (widget, event));
+}
+
 static void
 nautilus_location_entry_class_init (NautilusLocationEntryClass *class)
 {
+	GtkWidgetClass *widget_class;
 	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
 
+	widget_class = GTK_WIDGET_CLASS (class);
+
+	widget_class->focus_in_event = nautilus_location_entry_focus_in;
+
 	gobject_class = G_OBJECT_CLASS (class);
 	gobject_class->finalize = finalize;
 	
@@ -482,8 +516,11 @@ nautilus_location_entry_init (NautilusLocationEntry *entry)
 
 	nautilus_entry_set_special_tab_handling (NAUTILUS_ENTRY (entry), TRUE);
 
-	g_signal_connect_object (entry, "event_after",
-				 G_CALLBACK (editable_event_after_callback), entry, 0);
+	g_signal_connect (entry, "event_after",
+		          G_CALLBACK (editable_event_after_callback), entry);
+
+	g_signal_connect (entry, "notify::text",
+			  G_CALLBACK (nautilus_location_entry_text_changed), NULL);
 
 }
 
@@ -496,3 +533,18 @@ nautilus_location_entry_new (void)
 
 	return entry;
 }
+
+void
+nautilus_location_entry_set_special_text (NautilusLocationEntry *entry,
+					  const char            *special_text)
+{
+	entry->details->has_special_text = TRUE;
+	
+	g_free (entry->details->special_text);
+	entry->details->special_text = g_strdup (special_text);
+
+	entry->details->setting_special_text = TRUE;
+	gtk_entry_set_text (GTK_ENTRY (entry), special_text);
+	entry->details->setting_special_text = FALSE;
+}
+
