@@ -78,14 +78,12 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libnautilus-private/nautilus-customization-data.h>
 #include <libnautilus-private/nautilus-directory.h>
-#include <libnautilus-private/nautilus-drag-window.h>
 #include <libnautilus-private/nautilus-emblem-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-metadata.h>
-#include <libnautilus-private/nautilus-theme.h>
 #include <atk/atkrelationset.h>
 
 /* property types */
@@ -176,6 +174,8 @@ static void     nautilus_property_browser_hide_callback         (GtkWidget
 								 gpointer                       user_data);
 static void     nautilus_property_browser_drag_end              (GtkWidget                     *widget,
 								 GdkDragContext                *context);
+static void     nautilus_property_browser_drag_begin            (GtkWidget                     *widget,
+								 GdkDragContext                *context);
 static void     nautilus_property_browser_drag_data_get         (GtkWidget                     *widget,
 								 GdkDragContext                *context,
 								 GtkSelectionData              *selection_data,
@@ -183,6 +183,8 @@ static void     nautilus_property_browser_drag_data_get         (GtkWidget
 								 guint32                        time);
 static void     nautilus_property_browser_theme_changed         (gpointer                       user_data);
 static void     emit_emblems_changed_signal                     (void);
+static void     emblems_changed_callback                        (GObject                       *signaller,
+								 NautilusPropertyBrowser       *property_browser);
 
 /* misc utilities */
 static char *   strip_extension                                 (const char                    *string_to_strip);
@@ -191,6 +193,12 @@ static void     element_clicked_callback                        (GtkWidget
 								 const EelImageTableEvent *event,
 								 gpointer                       callback_data);
 
+static GdkPixbuf * make_drag_image                              (NautilusPropertyBrowser       *property_browser,
+								 const char                    *file_name);
+static GdkPixbuf * make_color_drag_image                        (NautilusPropertyBrowser       *property_browser,
+								 const char                    *color_spec,
+								 gboolean                       trim_edges);
+
 
 #define BROWSER_CATEGORIES_FILE_NAME "browser.xml"
 
@@ -232,6 +240,7 @@ nautilus_property_browser_class_init (GtkObjectClass *object_klass)
 	klass = NAUTILUS_PROPERTY_BROWSER_CLASS (object_klass);
 
 	object_klass->destroy = nautilus_property_browser_destroy;
+	widget_class->drag_begin = nautilus_property_browser_drag_begin;
 	widget_class->drag_data_get  = nautilus_property_browser_drag_data_get;
 	widget_class->drag_end  = nautilus_property_browser_drag_end;
 }
@@ -430,11 +439,12 @@ nautilus_property_browser_init (GtkObject *object)
 	g_signal_connect (property_browser, "hide",
 			  G_CALLBACK (nautilus_property_browser_hide_callback), NULL);
 
+	g_signal_connect_object (nautilus_signaller_get_current (),
+				 "emblems_changed",
+				 G_CALLBACK (emblems_changed_callback), property_browser, 0);
+
 	/* initially, display the top level */
 	nautilus_property_browser_set_path(property_browser, BROWSER_CATEGORIES_FILE_NAME);
-
-	/* Register that things may be dragged from this window */
-	nautilus_drag_window_register (GTK_WINDOW (property_browser));
 }
 
 /* Destroy the three dialogs for adding patterns/colors/emblems if any of them
@@ -560,6 +570,53 @@ nautilus_property_browser_set_drag_type (NautilusPropertyBrowser *property_brows
 	property_browser->details->drag_type = g_strdup (new_drag_type);
 }
 
+static void
+nautilus_property_browser_drag_begin (GtkWidget *widget,
+				      GdkDragContext *context)
+{
+	NautilusPropertyBrowser *property_browser;
+	GtkWidget *child;
+	GdkPixbuf *pixbuf;
+	int x_delta, y_delta;
+	char *element_name;
+
+	property_browser = NAUTILUS_PROPERTY_BROWSER (widget);
+
+	child = g_object_steal_data (G_OBJECT (property_browser), "dragged-image");
+	g_return_if_fail (child != NULL);
+
+	element_name = g_object_get_data (G_OBJECT (child), "property-name");
+	g_return_if_fail (child != NULL);
+
+	/* compute the offsets for dragging */
+	if (strcmp (drag_types[0].target, "application/x-color") != 0) {
+		/* it's not a color, so, for now, it must be an image */
+		/* fiddle with the category to handle the "reset" case properly */
+		char * save_category = property_browser->details->category;
+		if (eel_strcmp (property_browser->details->category, "colors") == 0) {
+			property_browser->details->category = "patterns";
+		}
+		pixbuf = make_drag_image (property_browser, element_name);
+		property_browser->details->category = save_category;
+	} else {
+		pixbuf = make_color_drag_image (property_browser, element_name, TRUE);
+	}
+
+        /* set the pixmap and mask for dragging */       
+	if (pixbuf != NULL) {
+		x_delta = gdk_pixbuf_get_width (pixbuf) / 2;
+		y_delta = gdk_pixbuf_get_height (pixbuf) / 2;
+
+		gtk_drag_set_icon_pixbuf
+			(context,
+			 pixbuf,
+			 x_delta, y_delta);
+		g_object_unref (pixbuf);
+	}
+
+}
+
+
 /* drag and drop data get handler */
 
 static void
@@ -691,7 +748,7 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 		} else {
 			icon_name = nautilus_emblem_get_icon_name_from_keyword (file_name);
 			pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL,
-									     NAUTILUS_ICON_SIZE_STANDARD,
+									     NAUTILUS_ICON_SIZE_STANDARD, TRUE,
 									     NULL);
 			g_free (icon_name);
 		}
@@ -902,7 +959,7 @@ remove_pattern(NautilusPropertyBrowser *property_browser, const char* pattern_na
 	if (gnome_vfs_unlink (pattern_uri) != GNOME_VFS_OK) {
 		char *message = g_strdup_printf (_("Sorry, but pattern %s couldn't be deleted."), pattern_name);
 		char *detail = _("Check that you have permission to delete the pattern.");
-		eel_show_error_dialog (message, detail, _("Couldn't Delete Pattern"), GTK_WINDOW (property_browser));
+		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
 	}
 	
@@ -932,7 +989,7 @@ remove_emblem (NautilusPropertyBrowser *property_browser, const char* emblem_nam
 	if (gnome_vfs_unlink (emblem_uri) != GNOME_VFS_OK) {
 		char *message = g_strdup_printf (_("Sorry, but emblem %s couldn't be deleted."), emblem_name);
 		char *detail = _("Check that you have permission to delete the emblem.");
-		eel_show_error_dialog (message, detail, _("Couldn't Delete Emblem"), GTK_WINDOW (property_browser));
+		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
 	}
 	else {
@@ -1097,7 +1154,7 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 			message = g_strdup (_("Sorry, but you did not supply a valid file name."));
 			detail = _("Please try again.");
 		}
-		eel_show_error_dialog (message, detail, _("Couldn't Install Pattern"), GTK_WINDOW (property_browser));
+		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
 		return;
 	}
@@ -1110,7 +1167,7 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 	if (basename && eel_strcmp (basename, RESET_IMAGE_NAME) == 0) {
 		eel_show_error_dialog (_("Sorry, but you can't replace the reset image."), 
 		                       _("Reset is a special image that cannot be deleted."), 
-		                       _("Not an Image"), NULL);
+		                       NULL);
 		g_free (path_uri);
 		g_free (basename);
 		return;
@@ -1142,7 +1199,7 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 	result = eel_copy_uri_simple (path_name, destination_name);		
 	if (result != GNOME_VFS_OK) {
 		char *message = g_strdup_printf (_("Sorry, but the pattern %s couldn't be installed."), path_name);
-		eel_show_error_dialog (message, NULL, _("Couldn't Install Pattern"), GTK_WINDOW (property_browser));
+		eel_show_error_dialog (message, NULL, GTK_WINDOW (property_browser));
 		g_free (message);
 	}
 		
@@ -1177,8 +1234,10 @@ add_new_pattern (NautilusPropertyBrowser *property_browser)
 static void
 add_color_to_file (NautilusPropertyBrowser *property_browser, const char *color_spec, const char *color_name)
 {
-	xmlNodePtr cur_node, new_color_node;
+	xmlNodePtr cur_node, new_color_node, children_node;
 	xmlDocPtr document;
+	xmlChar *child_color_name;
+	gboolean color_name_exists = FALSE;
 
 	document = read_browser_xml (property_browser);
 	if (document == NULL) {
@@ -1188,13 +1247,33 @@ add_color_to_file (NautilusPropertyBrowser *property_browser, const char *color_
 	/* find the colors category */
 	cur_node = get_color_category (document);
 	if (cur_node != NULL) {
-		/* add a new color node */
-		new_color_node = xmlNewChild (cur_node, NULL, "color", NULL);
-		xmlNodeSetContent (new_color_node, color_spec);
-		xmlSetProp (new_color_node, "local", "1");
-		xmlSetProp (new_color_node, "name", color_name);
+		/* check if theres already a color whith that name */
+		children_node = cur_node->xmlChildrenNode;
+		while (children_node != NULL) {
+			child_color_name = xmlGetProp (children_node, "name");	
+			if (xmlStrcmp (color_name, child_color_name) == 0) {
+				color_name_exists = TRUE;
+				xmlFree (child_color_name);
+				break;
+			}
+			xmlFree (child_color_name);
+
+			children_node = children_node->next;
+		}
 		
-		write_browser_xml (property_browser, document);
+		/* add a new color node */
+		if (!color_name_exists) {
+			new_color_node = xmlNewChild (cur_node, NULL, "color", NULL);
+			xmlNodeSetContent (new_color_node, color_spec);
+			xmlSetProp (new_color_node, "local", "1");
+			xmlSetProp (new_color_node, "name", color_name);
+				
+			write_browser_xml (property_browser, document);
+		} else {
+			eel_show_error_dialog (_("The color cannot be installed."),
+					       _("Sorry, but you must specify an unused color name for the new color."), 
+			                       GTK_WINDOW (property_browser));
+		}
 	}
 	
 	xmlFreeDoc (document);
@@ -1224,7 +1303,7 @@ add_color_to_browser (GtkWidget *widget, gint which_button, gpointer *data)
 		if (strlen (stripped_color_name) == 0) {
 			eel_show_error_dialog (_("The color cannot be installed."),
 					       _("Sorry, but you must specify a non-blank name for the new color."), 
-			                       _("Couldn't Install Color"), GTK_WINDOW (property_browser));
+			                       GTK_WINDOW (property_browser));
 		
 		} else {
 			add_color_to_file (property_browser, color_spec, stripped_color_name);
@@ -1314,7 +1393,7 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 				} else {
 					char *message = g_strdup_printf
 						(_("Sorry, but \"%s\" is not a usable image file."), emblem_path);
-					eel_show_error_dialog (_("The file is not an image."), message, _("Not an Image"), GTK_WINDOW (property_browser));
+					eel_show_error_dialog (_("The file is not an image."), message, GTK_WINDOW (property_browser));
 					g_free (message);
 					g_free (emblem_path);
 					return;
@@ -1330,7 +1409,7 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 		if (pixbuf == NULL) {
 			char *message = g_strdup_printf
 				(_("Sorry, but \"%s\" is not a usable image file."), property_browser->details->image_path);
-			eel_show_error_dialog (_("The file is not an image."), message, _("Not an Image"), GTK_WINDOW (property_browser));
+			eel_show_error_dialog (_("The file is not an image."), message, GTK_WINDOW (property_browser));
 			g_free (message);
 		}
 		
@@ -1346,11 +1425,12 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 						       stripped_keyword,
 						       stripped_keyword,
 						       GTK_WINDOW (property_browser));
+		if (pixbuf != NULL)
+			g_object_unref (pixbuf);
 
 		nautilus_emblem_refresh_list ();
 		
 		emit_emblems_changed_signal ();	
-		nautilus_property_browser_update_contents (property_browser);
 		
 		g_free (stripped_keyword);
 	}
@@ -1478,10 +1558,7 @@ element_clicked_callback (GtkWidget *image_table,
 	NautilusPropertyBrowser *property_browser;
 	GtkTargetList *target_list;	
 	GdkDragContext *context;
-	GdkPixbuf *pixbuf;
-	int x_delta, y_delta;
 	const char *element_name;
-	int scroll_offset_x, scroll_offset_y;
 	GdkDragAction action;
 
 	g_return_if_fail (EEL_IS_IMAGE_TABLE (image_table));
@@ -1513,6 +1590,8 @@ element_clicked_callback (GtkWidget *image_table,
 	target_list = gtk_target_list_new (drag_types, G_N_ELEMENTS (drag_types));
 	nautilus_property_browser_set_dragged_file(property_browser, element_name);
 	action = event->button == 3 ? GDK_ACTION_ASK : GDK_ACTION_MOVE | GDK_ACTION_COPY;
+
+	g_object_set_data (G_OBJECT (property_browser), "dragged-image", child);
 	
 	context = gtk_drag_begin (GTK_WIDGET (property_browser),
 				  target_list,
@@ -1520,37 +1599,7 @@ element_clicked_callback (GtkWidget *image_table,
 				  event->button,
 				  event->event);
 	gtk_target_list_unref (target_list);
-	
-	/* compute the offsets for dragging */
-	if (strcmp (drag_types[0].target, "application/x-color")) {
-		/* it's not a color, so, for now, it must be an image */
-		/* fiddle with the category to handle the "reset" case properly */
-		char * save_category = property_browser->details->category;
-		if (eel_strcmp (property_browser->details->category, "colors") == 0) {
-			property_browser->details->category = "patterns";
-		}
-		pixbuf = make_drag_image (property_browser, element_name);
-		property_browser->details->category = save_category;
-		x_delta = gdk_pixbuf_get_width (pixbuf) / 2;
-		y_delta = gdk_pixbuf_get_height (pixbuf) / 2;
-	} else {
-		pixbuf = make_color_drag_image (property_browser, element_name, TRUE);
-		gdk_window_get_position (GTK_VIEWPORT (image_table->parent)->bin_window,
-					 &scroll_offset_x,
-					 &scroll_offset_y);
-		x_delta = event->x - child->allocation.x - scroll_offset_x;
-		y_delta = event->y - child->allocation.y - scroll_offset_y;
-	}
 
-        /* set the pixmap and mask for dragging */       
-	if (pixbuf != NULL) {
-		gtk_drag_set_icon_pixbuf
-			(context,
-			 pixbuf,
-			 x_delta, y_delta);
-		g_object_unref (pixbuf);
-	}
-	
 	/* optionally (if the shift key is down) hide the property browser - it will later be destroyed when the drag ends */	
 	property_browser->details->keep_around = (event->state & GDK_SHIFT_MASK) == 0;
 	if (! property_browser->details->keep_around) {
@@ -1643,7 +1692,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 			}
 			object_name = nautilus_emblem_get_keyword_from_icon_name (icon_name);
 			object_pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL,
-										    NAUTILUS_ICON_SIZE_SMALL,
+										    NAUTILUS_ICON_SIZE_SMALL, TRUE,
 										    &object_label);
 			if (object_label == NULL) {
 				object_label = g_strdup (object_name);
@@ -1728,6 +1777,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 		object_pixbuf = gdk_pixbuf_new_from_file (path, NULL);
 		g_free (path);
 		property_image = labeled_image_new (_("Erase"), object_pixbuf, "erase", PANGO_SCALE_LARGE);
+		g_object_unref (object_pixbuf);
 		eel_labeled_image_set_fixed_image_height (EEL_LABELED_IMAGE (property_image), MAX_EMBLEM_HEIGHT);
 
 		gtk_container_add (GTK_CONTAINER (image_table), property_image);
@@ -1763,7 +1813,7 @@ add_reset_property (NautilusPropertyBrowser *property_browser)
 	
 	g_free (reset_image_file_name);
 
-	reset_image = labeled_image_new (NULL, reset_chit, RESET_IMAGE_NAME, PANGO_SCALE_MEDIUM);
+	reset_image = labeled_image_new ("Reset", reset_chit, RESET_IMAGE_NAME, PANGO_SCALE_MEDIUM);
 	gtk_container_add (GTK_CONTAINER (property_browser->details->content_table), reset_image);
 	eel_wrap_table_reorder_child (EEL_WRAP_TABLE (property_browser->details->content_table),
 					   reset_image,
@@ -2194,6 +2244,14 @@ nautilus_property_browser_set_path (NautilusPropertyBrowser *property_browser,
 	nautilus_property_browser_update_contents (property_browser);  	
 }
 
+static void
+emblems_changed_callback (GObject *signaller,
+			  NautilusPropertyBrowser *property_browser)
+{
+	nautilus_property_browser_update_contents (property_browser);
+}
+
+
 static void
 emit_emblems_changed_signal (void)
 {
