@@ -30,11 +30,13 @@
 
 #include "nautilus-desktop-window.h"
 #include "nautilus-main.h"
+#include "nautilus-window-private.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
+#include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
@@ -43,6 +45,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <stdlib.h>
@@ -60,10 +63,14 @@ struct NautilusShellDetails {
 static void     finalize                         (GObject              *shell);
 static void     corba_open_windows              (PortableServer_Servant  servant,
 						 const Nautilus_URIList *list,
+						 const CORBA_char       *startup_id,
 						 const CORBA_char       *geometry,
+						 CORBA_boolean           browser_window,
 						 CORBA_Environment      *ev);
 static void     corba_open_default_window       (PortableServer_Servant  servant,
+						 const CORBA_char       *startup_id,
 						 const CORBA_char       *geometry,
+						 CORBA_boolean           browser_window,
 						 CORBA_Environment      *ev);
 static void     corba_start_desktop             (PortableServer_Servant  servant,
 						 CORBA_Environment      *ev);
@@ -120,31 +127,52 @@ nautilus_shell_new (NautilusApplication *application)
 }
 
 static void
-open_window (NautilusShell *shell, const char *uri, const char *geometry)
+open_window (NautilusShell *shell, const char *uri, const char *startup_id,
+	     const char *geometry, gboolean browser_window)
 {
+	char *home_uri;
 	NautilusWindow *window;
 
-	window = nautilus_application_create_window (shell->details->application);
-
-	if (geometry != NULL) {
-		eel_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
-								      geometry,
-								      APPLICATION_WINDOW_MIN_WIDTH,
-								      APPLICATION_WINDOW_MIN_HEIGHT,
-								      FALSE);
-	}
-
-	if (uri == NULL) {
-		nautilus_window_go_home (window);
+	if (browser_window ||
+	    eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
+		window = nautilus_application_create_navigation_window (shell->details->application,
+									startup_id,
+									gdk_screen_get_default ());
+		if (uri == NULL) {
+			nautilus_window_go_home (window);
+		} else {
+			nautilus_window_go_to (window, uri);
+		}
 	} else {
-		nautilus_window_go_to (window, uri);
+		home_uri = NULL;
+		if (uri == NULL) {
+			home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+			uri = home_uri;
+		}
+		
+		window = nautilus_application_present_spatial_window (shell->details->application,
+								      NULL,
+								      startup_id,
+								      uri,
+								      gdk_screen_get_default ());
+		g_free (home_uri);
+	}
+	
+	if (geometry != NULL && !GTK_WIDGET_VISIBLE (window)) {
+		eel_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
+								 geometry,
+								 APPLICATION_WINDOW_MIN_WIDTH,
+								 APPLICATION_WINDOW_MIN_HEIGHT,
+								 FALSE);
 	}
 }
 
 static void
 corba_open_windows (PortableServer_Servant servant,
 		    const Nautilus_URIList *list,
+		    const CORBA_char *startup_id,
 		    const CORBA_char *geometry,
+		    CORBA_boolean browser_window,
 		    CORBA_Environment *ev)
 {
 	NautilusShell *shell;
@@ -155,13 +183,15 @@ corba_open_windows (PortableServer_Servant servant,
 	/* Open windows at each requested location. */
 	for (i = 0; i < list->_length; i++) {
 		g_assert (list->_buffer[i] != NULL);
-		open_window (shell, list->_buffer[i], geometry);
+		open_window (shell, list->_buffer[i], startup_id, geometry, browser_window);
 	}
 }
 
 static void
 corba_open_default_window (PortableServer_Servant servant,
+			   const CORBA_char *startup_id,
 			   const CORBA_char *geometry,
+			   CORBA_boolean browser_window,
 			   CORBA_Environment *ev)
 {
 	NautilusShell *shell;
@@ -170,7 +200,7 @@ corba_open_default_window (PortableServer_Servant servant,
 
 	if (!restore_window_states (shell)) {
 		/* Open a window pointing at the default location. */
-		open_window (shell, NULL, geometry);
+		open_window (shell, NULL, startup_id, geometry, browser_window);
 	}
 }
 
@@ -219,6 +249,7 @@ corba_quit (PortableServer_Servant servant,
 #define WINDOW_STATE_ATTRIBUTE_X	2
 #define WINDOW_STATE_ATTRIBUTE_Y	3
 #define WINDOW_STATE_ATTRIBUTE_LOCATION	4
+#define WINDOW_STATE_ATTRIBUTE_SCREEN	5
 
 static void
 save_window_states (void)
@@ -231,6 +262,7 @@ save_window_states (void)
 	int x, y, width, height;
 	char *location;
 	EelStringList *states;
+	int screen_num = -1;
 
 	states = NULL;
 	windows = nautilus_application_get_window_list ();
@@ -247,10 +279,14 @@ save_window_states (void)
 
 		location = nautilus_window_get_location (window);
 
-		window_attributes = g_strdup_printf ("%d,%d,%d,%d,%s", 
+		screen_num = gdk_screen_get_number (
+					gtk_window_get_screen (GTK_WINDOW (window)));
+
+		window_attributes = g_strdup_printf ("%d,%d,%d,%d,%s,%d", 
 						     width, height, 
 						     x, y, 
-						     location);
+						     location,
+						     screen_num);
 		g_free (location);
 		
 		if (states == NULL) {
@@ -260,7 +296,9 @@ save_window_states (void)
 		g_free (window_attributes);
 	}
 
-	eel_preferences_set_string_list (START_STATE_CONFIG, states);
+	if (eel_preferences_key_is_writable (START_STATE_CONFIG)) {
+		eel_preferences_set_string_list (START_STATE_CONFIG, states);
+	}
 
 	eel_string_list_free (states);
 }
@@ -277,6 +315,9 @@ restore_one_window_callback (const char *attributes,
 	int height;
 	char *location;
 	NautilusWindow *window;
+	GdkScreen *screen = NULL;
+	int screen_num;
+	int list_length;
 
 	g_return_if_fail (eel_strlen (attributes) > 0);
 	g_return_if_fail (NAUTILUS_IS_SHELL (callback_data));
@@ -285,17 +326,38 @@ restore_one_window_callback (const char *attributes,
 
 	attribute_list = eel_string_list_new_from_tokens (attributes, ",", TRUE);
 
+	list_length = eel_string_list_get_length (attribute_list);
+
 	eel_string_list_nth_as_integer (attribute_list, WINDOW_STATE_ATTRIBUTE_WIDTH, &width);
 	eel_string_list_nth_as_integer (attribute_list, WINDOW_STATE_ATTRIBUTE_HEIGHT, &height);
 	eel_string_list_nth_as_integer (attribute_list, WINDOW_STATE_ATTRIBUTE_X, &x);
 	eel_string_list_nth_as_integer (attribute_list, WINDOW_STATE_ATTRIBUTE_Y, &y);
 	location = eel_string_list_nth (attribute_list, WINDOW_STATE_ATTRIBUTE_LOCATION);
 
-	window = nautilus_application_create_window (shell->details->application);
-	
-	if (eel_strlen (location) > 0) {
-		nautilus_window_go_to (window, location);
+	/* Support sessions with no screen number for backwards compat.
+	 */
+	if (list_length >= WINDOW_STATE_ATTRIBUTE_SCREEN + 1) {
+		eel_string_list_nth_as_integer (
+			attribute_list, WINDOW_STATE_ATTRIBUTE_SCREEN, &screen_num);
+
+		screen = gdk_display_get_screen (gdk_display_get_default (), screen_num);
 	} else {
+		screen = gdk_screen_get_default ();
+	}
+
+#if NEW_UI_COMPLETE 
+/* don't always create object windows here */
+#endif
+	if (eel_strlen (location) > 0) {
+		window = nautilus_application_present_spatial_window (shell->details->application, 
+								      NULL,
+								      NULL,
+								      location,
+								      screen);
+	} else {
+		window = nautilus_application_create_navigation_window (shell->details->application,
+									NULL,
+									screen);
 		nautilus_window_go_home (window);
 	}
 
@@ -317,7 +379,9 @@ restore_window_states (NautilusShell *shell)
 	result = eel_string_list_get_length (states) > 0;
 	eel_string_list_for_each (states, restore_one_window_callback, shell);
 	eel_string_list_free (states);
-	eel_preferences_set_string_list (START_STATE_CONFIG, NULL);
+	if (eel_preferences_key_is_writable (START_STATE_CONFIG)) {
+		eel_preferences_set_string_list (START_STATE_CONFIG, NULL);
+	}
 	return result;
 }
 
